//            _ _              __  __ _      
//   __ _  __| | |_ _ __ __ _ / _|/ _(_) ___ 
//  / _` |/ _` | __| '__/ _` | |_| |_| |/ __|
// | (_| | (_| | |_| | | (_| |  _|  _| | (__ 
//  \__,_|\__,_|\__|_|  \__,_|_| |_| |_|\___|
//                                           
// Placement Excluder 23.1 (c) 2023 by R. Stinauer & R. Frank
//
// E: info@adtraffic.de | W: www.adtraffic.de

const SPREADSHEET_URL = 'YOUR_SPREADSHEET_URL';
const SHEET_NAME = 'placements';

function main(){
  var placementUrls = queryPlacementUrls();
  var queryListName = 'Globale Placement-Ausschlüsse';
  var excludedPlacements = getExcludedPlacements(queryListName);
  var whitelist = loadWhitelist(SPREADSHEET_URL,SHEET_NAME);
  var toBeExcludedPlacements = getToBeExcludedPlacements(placementUrls,excludedPlacements,whitelist);
  addExcludedPlacement(queryListName,toBeExcludedPlacements[0]);
  sendConfirmationEmail(queryListName,toBeExcludedPlacements);
}

function queryPlacementUrls(){
  var placementUrls = [];
  var query = 
    "SELECT " +
    "  group_placement_view.target_url " +
    "FROM group_placement_view " +
    "WHERE " +
    "  campaign.status IN ('ENABLED', 'PAUSED') " +
    "  AND campaign.advertising_channel_type IN ('DISPLAY') " +
    "  AND group_placement_view.placement_type IN ('WEBSITE') " +
    "  AND group_placement_view.target_url NOT REGEXP_MATCH '.*\.(de|at|ch|com|net|org|info)' " +
    "  AND campaign.name REGEXP_MATCH '^GDN_.*$' " +
    ""
    ;
  var result = AdsApp.search(query, {apiVersion: 'v12'});
  while (result.hasNext()) {
    var row = result.next();
    if ( placementUrls.indexOf(row['groupPlacementView']['targetUrl']) == -1 ) {
      placementUrls.push(row['groupPlacementView']['targetUrl']);
    }
  }
  return placementUrls.sort();
}

function getExcludedPlacements(queryListName){
  var excludedPlacements = [];
  var sharedExcludedPlacementList = getExcludedPlacementList(queryListName);
  var sharedExcludedPlacementIterator = sharedExcludedPlacementList.excludedPlacements().get();
  while (sharedExcludedPlacementIterator.hasNext()) {
    var sharedExcludedPlacement = sharedExcludedPlacementIterator.next();
    excludedPlacements.push(sharedExcludedPlacement.getUrl());
  }
  return excludedPlacements.sort();
}

function getExcludedPlacementList(queryListName){
  var sharedExcludedPlacementList = AdsApp.excludedPlacementLists()
      .withCondition("Name = '" + queryListName + "'").get().next();
  return sharedExcludedPlacementList;
}

function loadWhitelist(spreadsheetUrl,sheetName){
  const ss = SpreadsheetApp.openByUrl(spreadsheetUrl);
  const sheet = ss.getSheetByName(sheetName);
  const lastRow = sheet.getLastRow();
  const range = sheet.getRange(1, 1, lastRow);
  const values = range.getValues();
  var returnValues = [];
  for (i=0;i<values.length;i++){
    returnValues.push(values[i][0]);
  }
  return returnValues;
}

function getToBeExcludedPlacements(placementUrls,excludedPlacements,whitelist){
  var toBeExcludedPlacements = [];
  var numWhitelisted = 0;
  for (var i=0;i<placementUrls.length;i++){
      if (whitelist.indexOf(placementUrls[i]) != -1){
        numWhitelisted++;
      } else if (excludedPlacements.indexOf(placementUrls[i]) == -1){
          toBeExcludedPlacements.push(placementUrls[i]);
      }
  }
  return [toBeExcludedPlacements.sort(),numWhitelisted];
}

function addExcludedPlacement(queryListName,toBeExcludedPlacements){
  var sharedExcludedPlacementList = getExcludedPlacementList(queryListName);
  sharedExcludedPlacementList.addExcludedPlacements(toBeExcludedPlacements);
}

function sendConfirmationEmail(queryListName,toBeExcludedPlacements){
  var outputPlacements = "";
  for (i=0;i<toBeExcludedPlacements[0].length;i++){
      outputPlacements += toBeExcludedPlacements[0][i] + "\n";
  }
  var eMailAddress = "YOUR_EMAIL_ADDRESS(ES)";
  var eMailSubject = "Placement Excluder // Mein Account-Name";
  var eMailContent = toBeExcludedPlacements[0].length.toLocaleString("de") + " Placement(s) in Placement-Ausschlussliste " + queryListName + " ergänzt." + "\n" +
      toBeExcludedPlacements[1] + " Placement(s) übersprungen (whitelisted)." + "\n\n" + outputPlacements;
  console.log(eMailContent);
  sendSimpleTextEmail(eMailAddress,eMailSubject,eMailContent)
}

function sendSimpleTextEmail(eMailAddress,eMailSubject,eMailContent) {
  MailApp.sendEmail(eMailAddress,eMailSubject,eMailContent);
  Logger.log("Mail sent.");
}